#include "biasedfish.h"
#include "environ.h"


const int FORWARD_BIAS = 2;


// constructor

BiasedFish::BiasedFish(int id, const Position & pos)
  : MemoryFish(id, pos)                    // invoke the parent constructor
{}

// public accessing functions

Position BiasedFish::NextLocation(const Environment & env) {
  int dr = Location().Row() - OldLocation().Row();
  int dc = Location().Col() - OldLocation().Col();

  if (dr == 0 && dc == 0)    // no momentum
    return Fish::NextLocation(env);     // revert to uniform motion
  else {
    Position forward(Location().Row() + dr, Location().Col() + dc);
    Position left(Location().Row() + dc, Location().Col() - dr);
    Position right(Location().Row() - dc, Location().Col() + dr);

    Neighborhood choices;
    if (env.IsEmpty(forward))
      for (int k=0; k < FORWARD_BIAS; k++)
        choices.Add(forward);
    if (env.IsEmpty(left))
      choices.Add(left);
    if (env.IsEmpty(right))
      choices.Add(right);

    if (choices.Size() > 0)
      return choices.Select(randomVals.RandInt(0, choices.Size() - 1));
    else {
      // cannot move forward, left, or right; perhaps backward?
      Position reverse(Location().Row() - dr, Location().Col() - dc);
      if (env.IsEmpty(reverse))
        return reverse;
      else
        return Location();     // no move was possible; return current location
    }
  }
}
